<?php


namespace Service\Integration;


use Util\Utilities;

class GoogleDrive
{
    //"login": "javamailteste1@gmail.com",
    //"senha": "teknisateste"
    public $client;

    public function __construct()
    {
        $client = new \Google_Client();
        $client->setApplicationName('TeknisaDocs');
        $client->setScopes([\Google\Service\Docs::DOCUMENTS,\Google\Service\Drive::DRIVE]);
        $client->setAccessType('offline');
        $client->setAuthConfig('../config/credentialsIAM.json');
        $client->setPrompt('select_account consent');
        $this->client = $client;
    }

    public function saveDoc($fileName,$fileId){
        $client = $this->client;
        $driveService = new \Google\Service\Drive($client);

        $response = $driveService->files->get($fileId,array([
            'type' => null,
            'location' => null,
            'alt' => 'media',
        ],
            'fields' => '*'
        ));
        $arrayDirs = explode(DIRECTORY_SEPARATOR,$fileName);
        $nameBase = array_pop($arrayDirs);
        $extensions = explode('.',$nameBase);
        $nameBase = $extensions[0];

        $fileDir = implode(DIRECTORY_SEPARATOR,$arrayDirs);
        $name =  explode('.',$response->name);
        $extensionFinal = array_pop($name);

        $fileNameSave = $fileDir . DIRECTORY_SEPARATOR . $nameBase . '.' . $extensionFinal;
        $var = file_get_contents($response->webContentLink);
        file_put_contents($fileNameSave , $var);
        $driveService->files->delete($fileId);
        if($fileNameSave != $fileName){
            unlink($fileName);
        }
    }

    public function deleteFileDrive($fileId){
        $client = $this->client;
        $driveService = new \Google\Service\Drive($client);
        $driveService->files->delete($fileId);
    }

    public function uploadDoc($fileName){
        $client = $this->client;
        $driveService = new \Google\Service\Drive($client);

        $arrayDirs = explode(DIRECTORY_SEPARATOR,$fileName);
        $nameBase = array_pop($arrayDirs);
        $data = file_get_contents($fileName);


        $mimeType = Utilities::getTypeFile($fileName);
        if($data === false){
            throw New \Exception('Arquivo não encontrado!');
        }
        $document = $this->CreateDoc($nameBase, $mimeType, $driveService, $data);
        $fileId = $document->id;
        $OptParams = array(
            'fields' => '*'
        );

        $this->setPermissionAnyineWriter($driveService, $fileId, $OptParams);
        $urlDocGoogle = 'https://docs.google.com/document/d/' . $fileId . '/edit';
        return array('url' => $urlDocGoogle, 'id' => $fileId);
    }

    /**
     * @param string $nameBase
     * @param string $mimeType
     * @param \Google\Service\Drive $driveService
     * @param string $data
     * @return \Google\Service\Drive\DriveFile
     */
    public function CreateDoc(string $nameBase, string $mimeType, \Google\Service\Drive $driveService, string $data): \Google\Service\Drive\DriveFile
    {
        $fileMetadata = new \Google\Service\Drive\DriveFile(array(
            'name' => $nameBase));
        $fileMetadata->setMimeType($mimeType);
        $document = $driveService->files->create($fileMetadata, array([
            'mimeType' => $mimeType,
            'type' => "string",
            'location' => null,
            'uploadType' => 'multipart',
            'fields' => '*'
        ],
            'data' => $data));
        return $document;
    }

    /**
     * @param \Google\Service\Drive $driveService
     * @param string $fileId
     * @param array $OptParams
     */
    public function setPermissionAnyineWriter(\Google\Service\Drive $driveService, string $fileId, array $OptParams): void
    {
        $reponse = $driveService->files->get($fileId, $OptParams);
        foreach ($reponse->getPermissions() as $PermissionValue) {
            $updatedPerm = new \Google\Service\Drive\Permission(array(
                'type' => 'anyone',
                'role' => "writer",
                'additionalRoles' => [],
                'withLink' => true
            ));
            $driveService->permissions->create($fileId, $updatedPerm, array('fields' => 'id', 'supportsTeamDrives' => true));
        }
    }
}
