<?php
namespace Util\Mail;
use Teknisa\Libs\Util\Utilities;
use Util\Mail\Core\SmtpMail;
use Util\Messages;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zend\Mime\Part as ZendMimePart;
use Zend\Mime\Message as ZendMimeMessage;

class IssueMail {

    private $sender;
    private $smtpMail;
    private $baseDir = '';

    public function __construct(SmtpMail $smtpMail,$smtpUsername)
    {
        $this->sender   = $smtpUsername;
        $this->smtpMail = $smtpMail;
        $this->baseDir = __DIR__.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'EmailTemplates'.DIRECTORY_SEPARATOR;
    }

    public function getMailMainTemplatePath()
    {
        return $this->baseDir . 'issue_mail_main.html';
    }
    
    public function getMailBodyIssueTemplatePath()
    {
        return $this->baseDir . 'issue_mail_body_issue.html';
    }

    public function getMailBodyHomologatedTemplatePath()
    {
        return $this->baseDir . 'issue_mail_body_homologated.html';
    }

    public function getMailBodyVersionTemplatePath()
    {
        return $this->baseDir . 'issue_mail_body_version.html';
    }
    
    public function getMailBodyCommentTemplatePath()
    {
        return $this->baseDir . 'issue_mail_body_comment.html';
    }
    
    public function getMailBodyDescriptionTemplatePath()
    {
        return $this->baseDir . 'issue_mail_body_description.html';
    }
    
    public function getMailMainTemplate($fromDevOps = false, $fromDeploy = false)
    {
        $mainTemplate = $this->fileGetContentUtf8($this->getMailMainTemplatePath());
        $replaceable  = array("%issueData%", "%reference%", "%reporter%", "%actionDescription%", "%footerMessage%");
        if($fromDevOps){
            $toReplace    = array(Messages::MAIL_ISSUE_DATA, Messages::MAIL_REFERENCE, Messages::MAIL_REPORTER, Messages::MAIL_ACTION_DESCRIPTION_DEVOPS, Messages::MAIL_FOOTER_MESSAGE);
        } else if($fromDeploy){
            $toReplace    = array(Messages::MAIL_ISSUE_DATA, Messages::MAIL_REFERENCE, Messages::MAIL_REPORTER, Messages::MAIL_ACTION_DESCRIPTION_DEPLOY, Messages::MAIL_FOOTER_MESSAGE);
        } else{
            $toReplace    = array(Messages::MAIL_ISSUE_DATA, Messages::MAIL_REFERENCE, Messages::MAIL_REPORTER, Messages::MAIL_ACTION_DESCRIPTION, Messages::MAIL_FOOTER_MESSAGE);
        }
        return str_replace($replaceable, $toReplace, $mainTemplate);
    }
    
    public function buildBodyMessage(Entities\Task $task)
    {
        $replacedContents = $this->fileGetContentUtf8($this->getMailBodyHomologatedTemplatePath());
        $replaceable      = array('%taskName%', '%responseDescription%', '%description%', '%response%');
        $toReplace        = array($task->getName(), $task->getResponse(), Messages::MAIL_DESCRIPTION, Messages::MAIL_RESPONSE);
        return str_replace($replaceable, $toReplace, $replacedContents);
    }

    public function buildBodyMessageIssue($data)
    {
        $replacedContents = $this->fileGetContentUtf8($this->getMailBodyIssueTemplatePath());
        $replaceable      = array('%data%');
        $toReplace        = array($data);
        return str_replace($replaceable, $toReplace, $replacedContents);
    }

    public function buildBodyMessageVersion($version)
    {
        $replacedContents = $this->fileGetContentUtf8($this->getMailBodyVersionTemplatePath());
        $replaceable      = array('%product%', '%version%');
        $toReplace        = array($version['NAME'], $version['VERSION']);
        return str_replace($replaceable, $toReplace, $replacedContents);
    }
    
    public function buildEmailComment($comment)
    {
        $comment = $this->changeTagHtmlByEmail($comment);
        return str_replace("%comment%", $comment, $this->fileGetContentUtf8($this->getMailBodyCommentTemplatePath()));
    }
    
    public function buildEmailIssueDescription($issueDescription)
    {
        $replaceable = array("%issueDescription%", "%description%");
        $toReplace   = array($issueDescription, Messages::MAIL_DESCRIPTION);
        return str_replace($replaceable, $toReplace, $this->fileGetContentUtf8($this->getMailBodyDescriptionTemplatePath()));
    }
    
    public function buildSubject($issueId, $issueName, $subjectAction)
    {
        $replaceable = array("%issueId%", "%issueName%", "%subjectAction%");
        $toReplace = array($issueId, $issueName, $subjectAction);
        return str_replace($replaceable, $toReplace, Messages::MAIL_SUBJECT_DESCRIPTION);
    }
    
    public function sendCommentMail($to, $commenterName, $comment, $date, $issueTitle, $issueName = null, $reporterName = null, $sponsor = null, $issueId = null, $productName, $organizationName = null)
    {
        $subject           = $this->buildSubject($issueId, $issueName, Messages::MAIL_NEW_ISSUE_COMMENT);
        $mainTemplate      = $this->getMailMainTemplate();
        $commentTemplate   = $this->buildEmailComment($comment);
        $actionDescription = str_replace("%issueId%", $issueId, Messages::MAIL_ACTION_INCLUDED_NEW_COMMENT);
        $message           = str_replace("%innerContent%", $commentTemplate, $mainTemplate);
        $message           = $this->buildEmailHeader($message, $commenterName, $actionDescription, $productName, $date, $issueName, $reporterName, $organizationName);
        try{
            $this->encodeAndSend($to, $subject, $message);
            return true;
        }catch (\Exception $e){
            /*$subject         = $this->buildSubject($issueId, '', Messages::MAIL_NEW_ISSUE_COMMENT);
            $subject = str_replace('()', '', $subject);
            $this->encodeAndSend($to, $subject, $message);*/
            return false;
        }
    }

    public function sendReopentMail($to, $commenterName, $date, $issueTitle, $issueName, $reporterName, $sponsor, $issueId, $productName, $comment = null, $organizationName = null)
    {
        $subject           = $this->buildSubject($issueId, $issueName, Messages::MAIL_REOPEN_ISSUE_MESSAGE);
        $mainTemplate      = $this->getMailMainTemplate();

        $commentTemplate   = $this->buildEmailComment($comment);
        $actionDescription = str_replace("%issueId%", $issueId, Messages::MAIL_ACTION_REFUSED_ISSUE);
        $message           = str_replace("%innerContent%", $commentTemplate, $mainTemplate);
        $message           = $this->buildEmailHeader($message, $commenterName, $actionDescription, $productName, $date, $issueName, $reporterName, $organizationName);

        try{
            $this->encodeAndSend($to, $subject, $message);
        }catch (\Exception $e){
            $subject         = $this->buildSubject($issueId, '', Messages::MAIL_REOPEN_ISSUE_MESSAGE);
            $subject = str_replace('()', '', $subject);
            $this->encodeAndSend($to, $subject, $message);
        }
    }
    
    private function buildEmailHeader($message, $resourceName, $actionDescription, $productName, $date, $issueName, $reporterName, $organizationName, $version = '', $server = '', $availability = '')
    {
        $replaceable = array('%resourceName%', '%action%', '%productName%', '%date%', '%issueName%', '%reporterName%', '%organizationName%', '%version%', '%server%', '%isAvailable%');
        $toReplace   = array($resourceName, $actionDescription, $productName, $date, $issueName, $reporterName, $organizationName, $version, $server, $availability);
        return str_replace($replaceable, $toReplace, $message);
    }

    public function sendAvailableMail($to, $issueTitle, $commenterName, $date, $bodyTemplate, $version = '.',
                                      $reporterName, $sponsor, $issueName, $issueId, $productName,
                                      $description = null, $organizationName = null, $requestHTMLBuild = false, $bodyTemplateDescription = '',
                                      $fromDevOps = false, $fromDeploy = false, $server = '', $availability = '')
    {
        if($fromDeploy){
            $subject = $this->buildSubject($issueId, $issueName, Messages::MAIL_READY_ISSUE_SUBJECT);
        } else{
            $subject = $this->buildSubject($issueId, $issueName, Messages::MAIL_AVAILABLE_ISSUE_SUBJECT);
        }

        $mainTemplate    = $this->getMailMainTemplate($fromDevOps, $fromDeploy);

        if($description){
            if($requestHTMLBuild == true){
                $commentTemplate = $this->buildEmailComment($description);

            }else {
                $commentTemplate = $description;
            }
        }else{

            $commentTemplate = $this->buildEmailComment($description ? $description : (Messages::MAIL_WAS_RELEASED . $version));
        }
        if(!empty($bodyTemplateDescription)){
            $bodyTemplateDescription = "<tr><td class='title'><div>". Messages::MAIL_DESCRIPTION ."</div></td></tr>". $bodyTemplateDescription;
        }

        if(!empty($bodyTemplate)){
            $bodyTemplate = "<tr><td class='title'><div>". Messages::MAIL_RESPONSE ."</div></td></tr>". $bodyTemplate;
        }

        if($fromDevOps){
            $actionDescription = str_replace("%issueId%", $issueId, Messages::MAIL_ACTION_HOMOLOGATED_ISSUE);
            $message           = str_replace("%innerContent%", $commentTemplate.$bodyTemplateDescription.$bodyTemplate, $mainTemplate);
            $message = $this->buildEmailHeader($message, $commenterName, $actionDescription, $productName, $date, $issueName, $reporterName, $organizationName);
        } else if($fromDeploy){
            $actionDescription = str_replace("%issueId%", $issueId, Messages::MAIL_ACTION_UPDATED_ISSUE);
            $message           = str_replace("%innerContent%", $bodyTemplateDescription.$bodyTemplate, $mainTemplate);
            $message = $this->buildEmailHeader($message, $commenterName, $actionDescription, $productName, $date, $issueName, $reporterName, $organizationName, $version, $server);
        } else{
            $actionDescription = str_replace("%issueId%", $issueId, Messages::MAIL_ACTION_RELEASED_ISSUE) . ($version == '.' ?  '' : $version) ." ";
            $message           = str_replace("%innerContent%", $commentTemplate.$bodyTemplateDescription.$bodyTemplate, $mainTemplate);
            $message = $this->buildEmailHeader($message, $commenterName, $actionDescription, $productName, $date, $issueName, $reporterName, $organizationName, null, null, $availability);
        }

        try{
            $this->encodeAndSend($to, $subject, $message);
        }catch (\Exception $e){
            $subject         = $this->buildSubject($issueId, '', Messages::MAIL_AVAILABLE_ISSUE_SUBJECT);
            $subject = str_replace('()', '', $subject);
            $this->encodeAndSend($to, $subject, $message);
        }
    }

    public function sendOpenIssueMail($to, $issueTitle, $commenterName, $date, $description,  $reporterName, $sponsor, $issueName, $issueId, $productName, $organizationName = null)
    {
        $description = $this->changeTagHtmlByEmail($description);
        $subject         = $this->buildSubject($issueId, $issueName, Messages::MAIL_WAS_OPENED);
        $mainTemplate    = $this->getMailMainTemplate();
        $descriptionTemplate = $this->buildEmailIssueDescription($description);
        $actionDescription = str_replace("%issueId%", $issueId, Messages::MAIL_OPENED_ISSUE);
        $message           = str_replace("%innerContent%", $descriptionTemplate, $mainTemplate);
        $message = $this->buildEmailHeader($message, $commenterName, $actionDescription, $productName, $date, $issueName, $reporterName, $organizationName);

        try{
            $this->encodeAndSend($to, $subject, $message);
        }catch (\Exception $e){
            $subject         = $this->buildSubject($issueId, '', Messages::MAIL_WAS_OPENED);
            $subject = str_replace('()', '', $subject);
            $this->encodeAndSend($to, $subject, $message);
        }
    }
    
    private function encodeAndSend($to, $subject, $message)
    {
        $message        = utf8_decode($message);
        $mimePart       = new ZendMimePart($message);
        $mimeMessage    = new ZendMimeMessage();
        $mimePart->type = 'text/html';
        $mimeMessage->setParts(array($mimePart));
        $this->sendHtml($to, $subject, $mimeMessage);
    }

    public function sendIrregularitiesMail($email, $message)
    {
        $this->send($email, Messages::MAIL_IRREGULAR_OPERATION, $message);
    }

    private function send($to, $subject, $message)
    {
        $this->smtpMail->setFrom($this->sender);
        $this->smtpMail->setTo($to);
        $this->smtpMail->setSubject($subject);
        $this->smtpMail->setBody($message);
        $this->smtpMail->send();
    }

    private function sendHtml($to, $subject, $message)
    {
        $this->smtpMail->setFrom($this->sender);
        $this->smtpMail->setTo($to);
        $this->smtpMail->setSubject($subject);
        $this->smtpMail->setBody($message);
        $this->smtpMail->sendHtml();
    }

    function fileGetContentUtf8($fileName)
    {
        $content = file_get_contents($fileName);
        return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
    }
    private function changeTagHtmlByEmail($comment){
        $comment = str_replace("<", "&lt ", $comment);
        $comment = str_replace(">", " &gt", $comment);
        return $comment;
    }
} 